package jp.agentec.sinaburocast.service;

import jp.agentec.sinaburocast.common.util.SinaburoUtil;
import jp.agentec.sinaburocast.entity.AbstractEntity;

import org.apache.commons.lang.IllegalClassException;
import org.seasar.extension.jdbc.DbmsDialect;
import org.seasar.extension.jdbc.manager.JdbcManagerImplementor;
import org.seasar.extension.jdbc.service.S2AbstractService;

/**
 * AbstractServiceクラス
 *
 * @author tsukada
 *
 * @param <ENTITY>
 */
public abstract class AbstractService<ENTITY extends AbstractEntity> extends S2AbstractService<ENTITY> {
//	private final Log logger= LogFactory.getLog(getClass());

	/**
	 * PostgreSQLのシーケンスキーを指定し、次に採番されるシーケンス番号を取得します。
	 * @param sequenceName シーケンスキー
	 * @return Integer
	 */
	public Integer getSeqNextVal(String sequenceName) {
        DbmsDialect dialect = ((JdbcManagerImplementor)this.jdbcManager).getDialect();

        String nextValSql = dialect.getSequenceNextValString(sequenceName, 0);

        return this.jdbcManager.selectBySql(Integer.class, nextValSql).getSingleResult();
	}

	/**
	 * PostgreSQLのシーケンスキーを指定し、次に採番されるシーケンス番号を取得します。
	 * @param sequenceName シーケンスキー
	 * @return Long
	 */
	public Long getSeqNextLongVal(String sequenceName) {
        DbmsDialect dialect = ((JdbcManagerImplementor)this.jdbcManager).getDialect();

        String nextValSql = dialect.getSequenceNextValString(sequenceName, 0);

        return this.jdbcManager.selectBySql(Long.class, nextValSql).getSingleResult();
	}

	/**
	 * PostgreSQLのシーケンスキーを指定し、次に採番されるシーケンス番号を取得します。
	 * @param <T>
	 * @param valueClass
	 * @param sequenceName
	 * @return
	 */
	public <T> T getSeqNextVal(Class<T> valueClass, String sequenceName) {

        if (!(this.jdbcManager instanceof JdbcManagerImplementor)) {
            throw new IllegalClassException(
                JdbcManagerImplementor.class, this.jdbcManager);
        }

        DbmsDialect dialect = ((JdbcManagerImplementor)this.jdbcManager).getDialect();

        String nextValSql = dialect.getSequenceNextValString(sequenceName, 0);

        return this.jdbcManager.selectBySql(valueClass, nextValSql).getSingleResult();
    }

	/**
	 * DB共通項目を設定し、Insertします
	 * @param entity
	 * @param insIdStr ユーザーID
	 * @return
	 */
	public int insert(ENTITY entity, String insIdStr){
		setInsertMetaInfo(entity, insIdStr);
		return super.insert(entity);
	}

	/**
	 * DB共通項目を設定し、Updateします
	 * @param entity
	 * @param insId ユーザーID
	 * @return
	 */
	public int update(ENTITY entity, String updIdStr){
		setUpdateMetaInfo(entity, updIdStr);
		return super.update(entity);
	}

	/**
	 * 登録者・登録日・バージョンを設定する
	 * @param entity
	 * @param insId ユーザーID
	 */
	protected void setInsertMetaInfo(ENTITY entity, String insId){
		entity.insId = insId;
		entity.insertDate = SinaburoUtil.getTimestamp();
		entity.version = 0;
		setUpdateMetaInfo(entity,insId);
	}

	/**
	 * 更新者・更新日を設定する
	 * @param entity
	 * @param insId ユーザーID
	 */
	protected void setUpdateMetaInfo(ENTITY entity, String updateId) {
		entity.upId = updateId;
		entity.updateDate = SinaburoUtil.getTimestamp();
	}

}